import os
import numpy as np

import tempfile
os.environ['MPLCONFIGDIR'] = tempfile.mkdtemp()

# do this before importing pylab or pyplot
import matplotlib
matplotlib.use('Agg')

import matplotlib.pyplot as plt
from django.conf import settings

from ..utils import remove_file, get_color


def create_bar_charts(name, titles, series, legend):
    path = os.path.join(settings.MEDIA_ROOT, 'stats', name)
    remove_file(path)

    x = np.arange(len(titles))

    plt.xticks(range(len(titles)), titles)

    colors = [
        get_color("2A437DFF"),
        get_color("9398C2FF"),
        get_color("A9B3EAFF")
    ]
    plt.bar(x + 0.00, series[0], color=colors[0], width=0.25)
    plt.bar(x + 0.25, series[1], color=colors[1], width=0.25)
    plt.bar(x + 0.50, series[2], color=colors[2], width=0.25)

    plt.legend(legend)
    plt.savefig(path, dpi=400)
    plt.clf()


def create_age_plot(items, title):
    path = os.path.join(settings.MEDIA_ROOT, 'stats/ages.png')
    remove_file(path)

    x = []
    y = []
    for item in items:
        x.append(item['age'])
        y.append(item['sum'])

    plt.xlim(20, 100)
    plt.ylim(0, 150)
    plt.title(title)
    plt.scatter(x, y, color=get_color("9398C2FF"))
    plt.ylabel("Nombre d'inscrits")
    plt.xlabel("Age")
    plt.savefig(path, dpi=400)
    plt.clf()



from subprocess import run


SERVICE_STATUS_STOPPED = 0
SERVICE_STATUS_STARTED = 1
SERVICE_STATUS_ERROR = 2

class Service:
    def __init__(self, name):
        self.name = name

    def status(self):
        try:
            process = run(['systemctl', 'status', self.name], capture_output=True)
            output = process.stdout.decode()
            if "Active: active (running)" in output:
                return SERVICE_STATUS_STARTED
            else:
                return SERVICE_STATUS_STOPPED
        except OSError:
            return SERVICE_STATUS_ERROR

    def start(self):
        try:
            process = run(['sudo', 'systemctl', 'start', self.name])
            return process.returncode == 0
        except OSError:
            return False


    def stop(self):
        try:
            process = run(['sudo', 'systemctl', 'stop', self.name])
            return process.returncode == 0
        except OSError:
            return False


class MemoryUsage:
    def __init__(self, total, free, used, cache):
        self.total = total
        self.free = free
        self.used = used
        self.cache = cache


def get_memory():
    try:
        process = run(['top', '-bn1'], capture_output=True)
        output = process.stdout.decode()
        if "Mem :" in output:
            i = output.index('Mem :')
            output = output[i+5:]

            i = output.index(' total,')
            total = float(output[:i])
            i += len(' total,')
            output = output[i+1:]

            i = output.index(' free,')
            free = float(output[:i])
            i += len(' free,')
            output = output[i+1:]

            i = output.index(' used,')
            used = float(output[:i])
            i += len(' used,')
            output = output[i+1:]

            i = output.index(' buff')
            cache = float(output[:i])

            return MemoryUsage(total, free, used, cache)
    except OSError as error:
        print(error)

    return None




from django.contrib.admin.views.decorators import staff_member_required
from django.contrib.auth.decorators import login_required
from foledol.django.tools.table import TableColumn, TableView

from ..models import Grid


class GridTables(TableView):
    def __init__(self):
        super().__init__(Grid, [
            TableColumn('name', "Nom", sortable=True),
            TableColumn('table', "Table", method="table_as_str")
        ], path='django/grids', search=True, sort='name_asc')
        self.update = 'django:grid_update'
        self.create = 'django:grid_create'

    def select(self, grids, search, order_by):
        if len(search) > 0:
            grids = Grid.objects.filter(name__contains=search)
        if order_by == 'name_asc':
            return grids.order_by('name')
        elif order_by == 'name_desc':
            return grids.order_by('-name')
        return grids


@login_required
@staff_member_required
def grid_list(request):
    return GridTables().render(request)


def grid_renumber(grid):
    order = 10
    for column in grid.column_set.all().order_by('order'):
        if column.order != order:
            column.order = order
            column.save()
        order += 10









import os
from io import BytesIO

from django.conf import settings
from reportlab.pdfgen import canvas
from reportlab.lib.colors import Color
from reportlab.pdfbase.pdfmetrics import stringWidth

from foledol.django.utils import image_ratio


class ReportColumn:
    def __init__(self, width, label):
        self.width = width
        self.label = label


class ReportItem:
    def __init__(self, key, label, route=None, confirm=False, report=None):
        self.key = key
        self.label = label
        self.route = route
        self.confirm = confirm
        self.report = report


def build_report(builder):
    buffer = BytesIO()
    page = canvas.Canvas(buffer)
    builder(page)

    page.save()
    pdf = buffer.getvalue()
    buffer.close()
    return pdf


class Report:
    line_spacing_small = 10
    line_spacing_normal = 15
    line_spacing_large = 20
    line_padding = 5
    warning = Color(0.9, 0, 0, 0.7)
    list_color = Color(0.9, 0.9, 0.9)
    font_size = 8
    font = 'Helvetica'

    def cut(self, text, font, font_size, width):
        text_width = stringWidth(text, font, font_size)
        while text_width > width:
            text = text[:-1]
            text_width = stringWidth(text, font, font_size)
        return text

    def get_lines(self, text, font, font_size, width):
        words = text.split() if text else []
        lines = []
        line = None
        for word in words:
            new_line = line + " " + word if line else word
            line_width = stringWidth(new_line, font, font_size)
            if line_width < width or line is None:
                line = new_line
            else:
                lines.append(line)
                line = word
        if line is not None and len(line) > 0:
            lines.append(line)
        return lines

    def get_height(self, text, font, font_size, width, line_spacing=line_spacing_normal):
        lines_count = len(self.get_lines(text, font, font_size, width))
        return lines_count * line_spacing if lines_count > 1 else line_spacing

    def wrap(self, page, x, y, text, font, font_size, width, line_spacing=line_spacing_small):
        lines = self.get_lines(text, font, font_size, width)
        for line in lines:
            page.drawString(x, y, self.cut(line, font, font_size, width))
            y -= line_spacing
        return y

    def compute_height(self, columns, values, line_spacing=line_spacing_normal, font=None, font_size=8):
        i = 0
        max_height = 0
        font = font if font else self.font
        for column in columns:
            height = self.get_height(values[i], font, font_size, column.width, line_spacing)
            if height > max_height:
                max_height = height
            i += 1
        return max_height

    def draw_row(self, page, x, y, columns, values, font=None, font_size=8, column_padding=0):
        i = 0
        font = font if font else self.font
        for column in columns:
            self.wrap(page, x, y, values[i], font, font_size, column.width)
            x += column.width
            x += column_padding
            i += 1
        return x

    def draw_row_nowrap(self, page, x, y, columns, values, font=None, font_size=8, column_padding=0):
        i = 0
        font = font if font else self.font
        for column in columns:
            page.drawString(x, y, self.cut(values[i], font, font_size, column.width))
            x += column.width
            x += column_padding
            i += 1
        return x

    def draw_title(self, page, x, y, title):
        path = os.path.join(settings.STATIC_FILES, 'logo.jpeg')
        path_logo = path if os.path.isfile(path) else None
        if path_logo:
            ratio_logo = image_ratio(path_logo)
            page.drawImage(path_logo, x, y - 10, int(32 * ratio_logo), 32)
            x += 55
        page.drawString(x, y, title)


class PredefinedReport:
    def __init__(self, name, filename, label, filter, report):
        self.name = name
        self.filename = filename
        self.label = label
        self.filter = filter
        self.report = report



# Generated by Django 2.2 on 2023-02-10 14:47

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('django', '0009_auto_20230210_1430'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='Template',
            new_name='FormTemplate',
        ),
        migrations.RenameModel(
            old_name='TemplateItem',
            new_name='FormTemplateItem',
        ),
    ]
